@@ -1,25 +1,38 @@
 """
-.. versionadded:: 0.93
-
 Geopy can calculate geodesic distance between two points using the
-[Vincenty distance](https://en.wikipedia.org/wiki/Vincenty's_formulae) or
-[great-circle distance](https://en.wikipedia.org/wiki/Great-circle_distance)
-formulas, with a default of Vincenty available as the function
-`geopy.distance.distance`.
+`geodesic distance
+<https://en.wikipedia.org/wiki/Geodesics_on_an_ellipsoid>`_ or the
+`great-circle distance
+<https://en.wikipedia.org/wiki/Great-circle_distance>`_,
+with a default of the geodesic distance available as the function
+``geopy.distance.distance``.
 
 Great-circle distance (:class:`.great_circle`) uses a spherical model of
-the earth, using the average great-circle radius of 6372.795 kilometers,
-resulting in an error of up to about 0.5%. The radius value is stored in
-:const:`distance.EARTH_RADIUS`, so it can be customized
-(it should always be in kilometers, however).
+the earth, using the mean earth radius as defined by the International
+Union of Geodesy and Geophysics, (2\\ *a* + *b*)/3 = 6371.0087714150598
+kilometers approx 6371.009 km (for WGS-84), resulting in an error of up
+to about 0.5%. The radius value is stored in
+:const:`distance.EARTH_RADIUS`, so it can be customized (it should
+always be in kilometers, however).
 
-Vincenty distance (:class:`.vincenty`) uses a more accurate ellipsoidal model
-of the earth. This is the default distance formula, and is thus aliased as
-``distance.distance``. There are multiple popular ellipsoidal models, and
-which one will be the most accurate depends on where your points are located
-on the earth. The default is the WGS-84 ellipsoid, which is the most globally
-accurate. geopy includes a few other
-models in the distance.ELLIPSOIDS dictionary::
+The geodesic distance is the shortest distance on the surface of an
+ellipsoidal model of the earth.  The default algorithm uses the method
+is given by `Karney (2013)
+<https://doi.org/10.1007%2Fs00190-012-0578-z>`_ (:class:`.geodesic`);
+this is accurate to round-off and always converges.  An older
+*deprecated* method due to `Vincenty (1975)
+<https://en.wikipedia.org/wiki/Vincenty's_formulae>`_
+(:class:`.vincenty`) is also available; this is only accurate to 0.2 mm
+and the distance calculation fails to converge for nearly antipodal
+points.
+
+``geopy.distance.distance`` currently uses :class:`.geodesic`.
+
+There are multiple popular ellipsoidal models,
+and which one will be the most accurate depends on where your points are
+located on the earth.  The default is the WGS-84 ellipsoid, which is the
+most globally accurate.  geopy includes a few other models in the
+:const:`distance.ELLIPSOIDS` dictionary::
 
                   model             major (km)   minor (km)     flattening
     ELLIPSOIDS = {'WGS-84':        (6378.137,    6356.7523142,  1 / 298.257223563),
@@ -30,52 +43,65 @@ models in the distance.ELLIPSOIDS dictionary::
                   'GRS-67':        (6378.1600,   6356.774719,   1 / 298.25),
                   }
 
-Here's an example usage of distance.vincenty::
+Here are examples of ``distance.distance`` usage::
 
-    >>> from geopy.distance import vincenty
+    >>> from geopy import distance
     >>> newport_ri = (41.49008, -71.312796)
     >>> cleveland_oh = (41.499498, -81.695391)
-    >>> vincenty(newport_ri, cleveland_oh).miles
-    538.3904451566326
+    >>> print(distance.distance(newport_ri, cleveland_oh).miles)
+    538.39044536
 
-Using great-circle distance::
+    >>> wellington = (-41.32, 174.81)
+    >>> salamanca = (40.96, -5.50)
+    >>> print(distance.distance(wellington, salamanca).km)
+    19959.6792674
 
-    >>> from geopy.distance import great_circle
-    >>> newport_ri = (41.49008, -71.312796)
-    >>> cleveland_oh = (41.499498, -81.695391)
-    >>> great_circle(newport_ri, cleveland_oh).miles
-    537.1485284062816
+The second example above fails with :class:`.vincenty`.
 
-You can change the ellipsoid model used by the Vincenty formula like so::
+Using :class:`.great_circle` distance::
 
-    >>> distance.vincenty(ne, cl, ellipsoid='GRS-80').miles
+    >>> print(distance.great_circle(newport_ri, cleveland_oh).miles)
+    536.997990696
 
-The above model name will automatically be retrieved from the ELLIPSOIDS dictionary.
-Alternatively, you can specify the model values directly::
+You can change the ellipsoid model used by the geodesic formulas like so::
 
-    >>> distance.vincenty(ne, cl, ellipsoid=(6377., 6356., 1 / 297.)).miles
+    >>> ne, cl = newport_ri, cleveland_oh
+    >>> print(distance.geodesic(ne, cl, ellipsoid='GRS-80').miles)
+
+The above model name will automatically be retrieved from the
+:const:`distance.ELLIPSOIDS` dictionary. Alternatively, you can specify
+the model values directly::
+
+    >>> distance.geodesic(ne, cl, ellipsoid=(6377., 6356., 1 / 297.)).miles
 
 Distances support simple arithmetic, making it easy to do things like
 calculate the length of a path::
 
+    >>> from geopy import Nominatim
     >>> d = distance.distance
+    >>> g = Nominatim(user_agent="specify_your_app_name_here")
     >>> _, wa = g.geocode('Washington, DC')
     >>> _, pa = g.geocode('Palo Alto, CA')
-    >>> (d(ne, cl) + d(cl, wa) + d(wa, pa)).miles
-    3276.157156868931
+    >>> print((d(ne, cl) + d(cl, wa) + d(wa, pa)).miles)
+    3277.30439191
 
 """
 from __future__ import division
 
-from math import atan, tan, sin, cos, pi, sqrt, atan2, asin
-from geopy.units import radians
-from geopy import units, util
-from geopy.point import Point
-from geopy.compat import string_compare
+import warnings
+from math import asin, atan, atan2, cos, pi, sin, sqrt, tan
 
-# Average great-circle radius in kilometers, from Wikipedia.
-# Using a sphere with this radius results in an error of up to about 0.5%.
-EARTH_RADIUS = 6372.795
+from geographiclib.geodesic import Geodesic
+
+from geopy import units, util
+from geopy.compat import cmp, py3k, string_compare
+from geopy.point import Point
+from geopy.units import radians
+
+# IUGG mean earth radius in kilometers, from
+# https://en.wikipedia.org/wiki/Earth_radius#Mean_radius.  Using a
+# sphere with this radius results in an error of up to about 0.5%.
+EARTH_RADIUS = 6371.009
 
 # From http://www.movable-type.co.uk/scripts/LatLongVincenty.html:
 #   The most accurate and widely used globally-applicable model for the earth
@@ -94,9 +120,36 @@ ELLIPSOIDS = {
     'GRS-67':        (6378.1600, 6356.774719, 1 / 298.25)
 }
 
+
+def lonlat(x, y, z=0):
+    """
+    ``geopy.distance.distance`` accepts coordinates in ``(y, x)``/``(lat, lon)``
+    order, while some other libraries and systems might use
+    ``(x, y)``/``(lon, lat)``.
+
+    This function provides a convenient way to convert coordinates of the
+    ``(x, y)``/``(lon, lat)`` format to a :class:`geopy.point.Point` instance.
+
+    Example::
+
+        >>> from geopy.distance import lonlat, distance
+        >>> newport_ri_xy = (-71.312796, 41.49008)
+        >>> cleveland_oh_xy = (-81.695391, 41.499498)
+        >>> print(distance(lonlat(*newport_ri_xy), lonlat(*cleveland_oh_xy)).miles)
+        538.3904453677203
+
+    :param x: longitude
+    :param y: latitude
+    :param z: (optional) altitude
+    :return: Point(latitude, longitude, altitude)
+    """
+    return Point(y, x, z)
+
+
 class Distance(object):
     """
-    Base for :class:`.great_circle` and :class:`.vincenty`.
+    Base for :class:`.great_circle`, :class:`.vincenty`, and
+    :class:`.geodesic`.
     """
 
     def __init__(self, *args, **kwargs):
@@ -147,69 +200,89 @@ class Distance(object):
     __bool__ = __nonzero__
 
     def measure(self, a, b):
+        """
+        Abstract method for measure
+        """
         raise NotImplementedError()
 
-    def __repr__(self): # pragma: no cover
+    def __repr__(self):  # pragma: no cover
         return 'Distance(%s)' % self.kilometers
 
-    def __str__(self): # pragma: no cover
+    def __str__(self):  # pragma: no cover
         return '%s km' % self.__kilometers
 
-    def __cmp__(self, other):
+    def __cmp__(self, other):  # py2 only
         if isinstance(other, Distance):
             return cmp(self.kilometers, other.kilometers)
         else:
             return cmp(self.kilometers, other)
 
+    if py3k:
+        def __eq__(self, other):
+            return self.__cmp__(other) == 0
+
+        def __ne__(self, other):
+            return self.__cmp__(other) != 0
+
+        def __gt__(self, other):
+            return self.__cmp__(other) > 0
+
+        def __lt__(self, other):
+            return self.__cmp__(other) < 0
+
+        def __ge__(self, other):
+            return self.__cmp__(other) >= 0
+
+        def __le__(self, other):
+            return self.__cmp__(other) <= 0
+
     @property
-    def kilometers(self): # pylint: disable=C0111
+    def kilometers(self):
         return self.__kilometers
 
     @property
-    def km(self): # pylint: disable=C0111
+    def km(self):
         return self.kilometers
 
     @property
-    def meters(self): # pylint: disable=C0111
+    def meters(self):
         return units.meters(kilometers=self.kilometers)
 
     @property
-    def m(self): # pylint: disable=C0111
+    def m(self):
         return self.meters
 
     @property
-    def miles(self): # pylint: disable=C0111
+    def miles(self):
         return units.miles(kilometers=self.kilometers)
 
     @property
-    def mi(self): # pylint: disable=C0111
+    def mi(self):
         return self.miles
 
     @property
-    def feet(self): # pylint: disable=C0111
+    def feet(self):
         return units.feet(kilometers=self.kilometers)
 
     @property
-    def ft(self): # pylint: disable=C0111
+    def ft(self):
         return self.feet
 
     @property
-    def nautical(self): # pylint: disable=C0111
+    def nautical(self):
         return units.nautical(kilometers=self.kilometers)
 
     @property
-    def nm(self): # pylint: disable=C0111
+    def nm(self):
         return self.nautical
 
 
 class great_circle(Distance):
     """
     Use spherical geometry to calculate the surface distance between two
-    geodesic points. This formula can be written many different ways,
-    including just the use of the spherical law of cosines or the haversine
-    formula.
+    points.
 
-    Set which radius of the earth to use by specifying a 'radius' keyword
+    Set which radius of the earth to use by specifying a ``radius`` keyword
     argument. It must be in kilometers. The default is to use the module
     constant `EARTH_RADIUS`, which uses the average great-circle radius.
 
@@ -218,8 +291,8 @@ class great_circle(Distance):
         >>> from geopy.distance import great_circle
         >>> newport_ri = (41.49008, -71.312796)
         >>> cleveland_oh = (41.499498, -81.695391)
-        >>> great_circle(newport_ri, cleveland_oh).miles
-        537.1485284062816
+        >>> print(great_circle(newport_ri, cleveland_oh).miles)
+        536.997990696
 
     """
 
@@ -246,7 +319,7 @@ class great_circle(Distance):
 
         return self.RADIUS * d
 
-    def destination(self, point, bearing, distance=None): # pylint: disable=W0621
+    def destination(self, point, bearing, distance=None):
         """
         TODO docs.
         """
@@ -275,11 +348,115 @@ class great_circle(Distance):
         return Point(units.degrees(radians=lat2), units.degrees(radians=lng2))
 
 
+GreatCircleDistance = great_circle
+
+
+class geodesic(Distance):
+    """
+    Calculate the geodesic distance between two points.
+
+    Set which ellipsoidal model of the earth to use by specifying an
+    ``ellipsoid`` keyword argument. The default is 'WGS-84', which is the
+    most globally accurate model.  If ``ellipsoid`` is a string, it is
+    looked up in the `ELLIPSOIDS` dictionary to obtain the major and minor
+    semiaxes and the flattening. Otherwise, it should be a tuple with those
+    values.  See the comments above the `ELLIPSOIDS` dictionary for
+    more information.
+
+    Example::
+
+        >>> from geopy.distance import geodesic
+        >>> newport_ri = (41.49008, -71.312796)
+        >>> cleveland_oh = (41.499498, -81.695391)
+        >>> print(geodesic(newport_ri, cleveland_oh).miles)
+        538.390445368
+
+
+    .. versionadded:: 1.13.0
+    """
+
+    ellipsoid_key = None
+    ELLIPSOID = None
+    geod = None
+
+    def __init__(self, *args, **kwargs):
+        self.set_ellipsoid(kwargs.pop('ellipsoid', 'WGS-84'))
+        if 'iterations' in kwargs:
+            warnings.warn('Ignoring unused `iterations` kwarg for geodesic '
+                          'distance.', DeprecationWarning, stacklevel=2)
+        kwargs.pop('iterations', 0)
+        major, minor, f = self.ELLIPSOID
+        super(geodesic, self).__init__(*args, **kwargs)
+
+    def set_ellipsoid(self, ellipsoid):
+        """
+        Change the ellipsoid used in the calculation.
+        """
+        if not isinstance(ellipsoid, (list, tuple)):
+            try:
+                self.ELLIPSOID = ELLIPSOIDS[ellipsoid]
+                self.ellipsoid_key = ellipsoid
+            except KeyError:
+                raise Exception(
+                    "Invalid ellipsoid. See geopy.distance.ELLIPSOIDS"
+                )
+        else:
+            self.ELLIPSOID = ellipsoid
+            self.ellipsoid_key = None
+        return
+
+    # Call geographiclib routines for measure and destination
+    def measure(self, a, b):
+        a, b = Point(a), Point(b)
+        lat1, lon1 = a.latitude, a.longitude
+        lat2, lon2 = b.latitude, b.longitude
+
+        if not (isinstance(self.geod, Geodesic) and
+                self.geod.a == self.ELLIPSOID[0] and
+                self.geod.f == self.ELLIPSOID[2]):
+            self.geod = Geodesic(self.ELLIPSOID[0], self.ELLIPSOID[2])
+
+        s12 = self.geod.Inverse(lat1, lon1, lat2, lon2,
+                                Geodesic.DISTANCE)['s12']
+
+        return s12
+
+    def destination(self, point, bearing, distance=None):
+        """
+        TODO docs.
+        """
+        point = Point(point)
+        lat1 = point.latitude
+        lon1 = point.longitude
+        azi1 = bearing
+
+        if distance is None:
+            distance = self
+        if isinstance(distance, Distance):
+            distance = distance.kilometers
+
+        if not (isinstance(self.geod, Geodesic) and
+                self.geod.a == self.ELLIPSOID[0] and
+                self.geod.f == self.ELLIPSOID[2]):
+            self.geod = Geodesic(self.ELLIPSOID[0], self.ELLIPSOID[2])
+
+        r = self.geod.Direct(lat1, lon1, azi1, distance,
+                             Geodesic.LATITUDE | Geodesic.LONGITUDE)
+
+        return Point(r['lat2'], r['lon2'])
+
+
+GeodesicDistance = geodesic
+
+
 class vincenty(Distance):
     """
-    Calculate the geodesic distance between two points using the formula
-    devised by Thaddeus Vincenty, with an accurate ellipsoidal model of the
-    earth.
+    .. deprecated:: 1.13
+       Use :class:`.geodesic` instead.
+       Vincenty will be removed in geopy 2.0.
+
+    Calculate the geodesic distance between two points using the Vincenty's
+    method.
 
     Set which ellipsoidal model of the earth to use by specifying an
     ``ellipsoid`` keyword argument. The default is 'WGS-84', which is the
@@ -294,24 +471,29 @@ class vincenty(Distance):
         >>> from geopy.distance import vincenty
         >>> newport_ri = (41.49008, -71.312796)
         >>> cleveland_oh = (41.499498, -81.695391)
-        >>> vincenty(newport_ri, cleveland_oh).miles
-        538.3904451566326
+        >>> print(vincenty(newport_ri, cleveland_oh).miles)
+        538.390445362
+
+    Note: Vincenty's method for distance fails to converge for some
+    valid (nearly antipodal) points. In such cases, use
+    :class:`.geodesic` which always produces an accurate result.
 
-    Note: This implementation of Vincenty distance fails to converge for
-    some valid points. In some cases, a result can be obtained by increasing
-    the number of iterations (`iterations` keyword argument, given in the
-    class `__init__`, with a default of 20). It may be preferable to use
-    :class:`.great_circle`, which is marginally less accurate, but always
-    produces a result.
     """
 
     ellipsoid_key = None
     ELLIPSOID = None
+    _show_deprecation_warning = True
 
     def __init__(self, *args, **kwargs):
+        if self._show_deprecation_warning:
+            warnings.warn('Vincenty is deprecated and is going to be removed '
+                          'in geopy 2.0. Use `geopy.distance.geodesic` '
+                          '(or the default `geopy.distance.distance`) '
+                          'instead, which is more accurate and always converges.',
+                          DeprecationWarning, stacklevel=2)
         self.set_ellipsoid(kwargs.pop('ellipsoid', 'WGS-84'))
         self.iterations = kwargs.pop('iterations', 20)
-        major, minor, f = self.ELLIPSOID # pylint: disable=W0612
+        major, minor, f = self.ELLIPSOID
         super(vincenty, self).__init__(*args, **kwargs)
 
     def set_ellipsoid(self, ellipsoid):
@@ -355,7 +537,9 @@ class vincenty(Distance):
         iter_limit = self.iterations
 
         i = 0
-        while abs(lambda_lng - lambda_prime) > 10e-12 and i <= iter_limit:
+
+        while (i == 0 or
+               (abs(lambda_lng - lambda_prime) > 10e-12 and i <= iter_limit)):
             i += 1
 
             sin_lambda_lng, cos_lambda_lng = sin(lambda_lng), cos(lambda_lng)
@@ -367,7 +551,7 @@ class vincenty(Distance):
             )
 
             if sin_sigma == 0:
-                return 0 # Coincident points
+                return 0  # Coincident points
 
             cos_sigma = (
                 sin_reduced1 * sin_reduced2 +
@@ -386,7 +570,7 @@ class vincenty(Distance):
                     sin_reduced1 * sin_reduced2 / cos_sq_alpha
                 )
             else:
-                cos2_sigma_m = 0.0 # Equatorial line
+                cos2_sigma_m = 0.0  # Equatorial line
 
             C = f / 16. * cos_sq_alpha * (4 + f * (4 - 3 * cos_sq_alpha))
 
@@ -429,7 +613,7 @@ class vincenty(Distance):
         s = minor * A * (sigma - delta_sigma)
         return s
 
-    def destination(self, point, bearing, distance=None): # pylint: disable=W0621
+    def destination(self, point, bearing, distance=None):
         """
         TODO docs.
         """
@@ -472,9 +656,10 @@ class vincenty(Distance):
             delta_sigma = B * sin_sigma * (
                 cos2_sigma_m + B / 4. * (
                     cos_sigma * (
-                        -1 + 2 * cos2_sigma_m
+                        -1 + 2 * cos2_sigma_m ** 2
                     ) - B / 6. * cos2_sigma_m * (
-                        -3 + 4 * sin_sigma ** 2) * (
+                        -3 + 4 * sin_sigma ** 2
+                    ) * (
                         -3 + 4 * cos2_sigma_m ** 2
                     )
                 )
@@ -516,7 +701,7 @@ class vincenty(Distance):
         return Point(units.degrees(radians=lat2), units.degrees(radians=lng2))
 
 
-# Set the default distance formula to the most generally accurate.
+VincentyDistance = vincenty
 
-distance = VincentyDistance = vincenty
-GreatCircleDistance = great_circle
+# Set the default distance formula
+distance = GeodesicDistance
